@@ -15,14 +15,17 @@
 from __future__ import annotations
 
 from abc import ABC, abstractmethod
+from typing import Any, Optional
 
 import gpflow
 import tensorflow as tf
 from gpflow.models import GPModel
+from gpflow.posteriors import BasePosterior, PrecomputeCacheType
+from gpflow.utilities.traversal import _merge_leaf_components, leaf_components
 from typing_extensions import Protocol
 
+from ... import logging
 from ...data import Dataset
-from ...logging import get_step_number, get_tensorboard_writer
 from ...types import TensorType
 from ..interfaces import (
     HasReparamSampler,
@@ -46,25 +49,56 @@ class GPflowPredictor(
             :class:`~trieste.models.optimizer.Optimizer` with :class:`~gpflow.optimizers.Scipy`.
         """
         if optimizer is None:
-            optimizer = Optimizer(gpflow.optimizers.Scipy())
+            optimizer = Optimizer(gpflow.optimizers.Scipy(), compile=True)
 
         self._optimizer = optimizer
+        self._posterior: Optional[BasePosterior] = None
 
     @property
     def optimizer(self) -> Optimizer:
         """The optimizer with which to train the model."""
         return self._optimizer
 
+    def create_posterior_cache(self) -> None:
+        """
+        Create a posterior cache for fast sequential predictions.  Note that this must happen
+        at initialisation and *after* we ensure the model data is variable. Furthermore,
+        the cache must be updated whenever the underlying model is changed.
+        """
+        self._posterior = self.model.posterior(PrecomputeCacheType.VARIABLE)
+
+    def __setstate__(self, state: dict[str, Any]) -> None:
+        # when unpickling we may need to regenerate the posterior cache
+        self.__dict__.update(state)
+        if self._posterior is not None:
+            self.create_posterior_cache()
+
+    def update_posterior_cache(self) -> None:
+        """Update the posterior cache. This needs to be called whenever the underlying model
+        is changed."""
+        if self._posterior is not None:
+            self._posterior.update_cache()
+
     @property
     @abstractmethod
     def model(self) -> GPModel:
         """The underlying GPflow model."""
 
     def predict(self, query_points: TensorType) -> tuple[TensorType, TensorType]:
-        return self.model.predict_f(query_points)
+        mean, cov = (self._posterior or self.model).predict_f(query_points)
+        # posterior predict can return negative variance values [cf GPFlow issue #1813]
+        if self._posterior is not None:
+            cov = tf.clip_by_value(cov, 1e-12, cov.dtype.max)
+        return mean, cov
 
     def predict_joint(self, query_points: TensorType) -> tuple[TensorType, TensorType]:
-        return self.model.predict_f(query_points, full_cov=True)
+        mean, cov = (self._posterior or self.model).predict_f(query_points, full_cov=True)
+        # posterior predict can return negative variance values [cf GPFlow issue #1813]
+        if self._posterior is not None:
+            cov = tf.linalg.set_diag(
+                cov, tf.clip_by_value(tf.linalg.diag_part(cov), 1e-12, cov.dtype.max)
+            )
+        return mean, cov
 
     def sample(self, query_points: TensorType, num_samples: int) -> TensorType:
         return self.model.predict_f_samples(query_points, num_samples)
@@ -80,6 +114,14 @@ class GPflowPredictor(
         """
         return self.model.kernel
 
+    def get_mean_function(self) -> gpflow.mean_functions.MeanFunction:
+        """
+        Return the mean function of the model.
+
+        :return: The mean function.
+        """
+        return self.model.mean_function
+
     def get_observation_noise(self) -> TensorType:
         """
         Return the variance of observation noise for homoscedastic likelihoods.
@@ -106,16 +148,18 @@ class GPflowPredictor(
         """
         Log model-specific information at a given optimization step.
         """
-        summary_writer = get_tensorboard_writer()
+        summary_writer = logging.get_tensorboard_writer()
         if summary_writer:
-            with summary_writer.as_default(step=get_step_number()):
-                tf.summary.scalar("kernel.variance", self.get_kernel().variance)
-                lengthscales = self.get_kernel().lengthscales
-                if tf.rank(lengthscales) == 0:
-                    tf.summary.scalar("kernel.lengthscale", lengthscales)
-                elif tf.rank(lengthscales) == 1:
-                    for i, lengthscale in enumerate(lengthscales):
-                        tf.summary.scalar(f"kernel.lengthscale.{i}", lengthscale)
+            with summary_writer.as_default(step=logging.get_step_number()):
+                kernel = self.get_kernel()
+                components = _merge_leaf_components(leaf_components(kernel))
+                for k, v in components.items():
+                    if v.trainable:
+                        if tf.rank(v) == 0:
+                            logging.scalar(f"kernel.{k}", v)
+                        elif tf.rank(v) == 1:
+                            for i, vi in enumerate(v):
+                                logging.scalar(f"kernel.{k}[{i}]", vi)
 
     def reparam_sampler(self, num_samples: int) -> ReparametrizationSampler[GPflowPredictor]:
         """
