@@ -18,6 +18,9 @@ GPflow wrappers.
 
 from __future__ import annotations
 
+from abc import ABC, abstractmethod
+from typing import Callable, Optional, Tuple, TypeVar, Union, cast
+
 import tensorflow as tf
 import tensorflow_probability as tfp
 
@@ -27,6 +30,7 @@ except (ModuleNotFoundError, ImportError):
     # temporary support for gpflux 0.2.3
     from gpflux.layers.basis_functions import RandomFourierFeatures as RFF
 
+from gpflux.math import compute_A_inv_b
 from typing_extensions import Protocol, runtime_checkable
 
 from ...types import TensorType
@@ -34,9 +38,11 @@ from ...utils import DEFAULTS, flatten_leading_dims
 from ..interfaces import (
     ProbabilisticModel,
     ReparametrizationSampler,
+    SupportsGetInducingVariables,
+    SupportsGetInternalData,
     SupportsGetKernel,
+    SupportsGetMeanFunction,
     SupportsGetObservationNoise,
-    SupportsInternalData,
     SupportsPredictJoint,
     TrajectoryFunction,
     TrajectoryFunctionClass,
@@ -191,30 +197,160 @@ class BatchReparametrizationSampler(ReparametrizationSampler[SupportsPredictJoin
 
 
 @runtime_checkable
-class SupportsGetKernelObservationNoiseInternalData(
-    SupportsGetKernel, SupportsGetObservationNoise, SupportsInternalData, Protocol
+class FeatureDecompositionInternalDataModel(
+    SupportsGetKernel,
+    SupportsGetMeanFunction,
+    SupportsGetObservationNoise,
+    SupportsGetInternalData,
+    Protocol,
 ):
     """
-    A probabilistic model that supports get_kernel, get_observation noise
-    and get_internal_data.
+    A probabilistic model that supports get_kernel, get_mean_function, get_observation_noise
+    and get_internal_data methods.
     """
 
     pass
 
 
-class RandomFourierFeatureTrajectorySampler(
-    TrajectorySampler[SupportsGetKernelObservationNoiseInternalData]
+@runtime_checkable
+class FeatureDecompositionInducingPointModel(
+    SupportsGetKernel, SupportsGetMeanFunction, SupportsGetInducingVariables, Protocol
+):
+    """
+    A probabilistic model that supports get_kernel, get_mean_function
+    and get_inducing_point methods.
+    """
+
+    pass
+
+
+FeatureDecompositionTrajectorySamplerModel = Union[
+    FeatureDecompositionInducingPointModel,
+    FeatureDecompositionInternalDataModel,
+]
+
+FeatureDecompositionTrajectorySamplerModelType = TypeVar(
+    "FeatureDecompositionTrajectorySamplerModelType",
+    bound=FeatureDecompositionTrajectorySamplerModel,
+    contravariant=True,
+)
+
+
+class FeatureDecompositionTrajectorySampler(
+    TrajectorySampler[FeatureDecompositionTrajectorySamplerModelType],
+    ABC,
 ):
     r"""
-    This class builds functions that approximate a trajectory sampled from an underlying Gaussian
-    process model. For tractibility, the Gaussian process is approximated with a Bayesian
-    Linear model across a set of features sampled from the Fourier feature decomposition of
-    the model's kernel. See :cite:`hernandez2014predictive` for details.
+
+    This is a general class to build functions that approximate a trajectory sampled from an
+    underlying Gaussian process model.
+
+    In particular, we approximate the Gaussian processes' posterior samples as the finite feature
+    approximation
+
+    .. math:: \hat{f}(x) = \sum_{i=1}^m \phi_i(x)\theta_i
+
+    where :math:`\phi_i` are m features and :math:`\theta_i` are feature weights sampled from a
+    given distribution
 
     Achieving consistency (ensuring that the same sample draw for all evalutions of a particular
     trajectory function) for exact sample draws from a GP is prohibitively costly because it scales
     cubically with the number of query points. However, finite feature representations can be
     evaluated with constant cost regardless of the required number of queries.
+    """
+
+    def __init__(
+        self,
+        model: FeatureDecompositionTrajectorySamplerModelType,
+        feature_functions: ResampleableRandomFourierFeatureFunctions,
+    ):
+        """
+        :param sample_size: The desired number of samples.
+        :param model: The model to sample from.
+        :raise ValueError: If ``dataset`` is empty.
+        """
+
+        super().__init__(model)
+        self._feature_functions = feature_functions
+        self._weight_sampler: Optional[Callable[[int], TensorType]] = None  # lazy init
+        self._mean_function = model.get_mean_function()
+
+    def __repr__(self) -> str:
+        """"""
+        return f"""{self.__class__.__name__}(
+        {self._model!r},
+        {self._feature_functions!r})
+        """
+
+    def get_trajectory(self) -> TrajectoryFunction:
+        """
+        Generate an approximate function draw (trajectory) by sampling weights
+        and evaluating the feature functions.
+
+        :return: A trajectory function representing an approximate trajectory from the Gaussian
+            process, taking an input of shape `[N, D]` and returning shape `[N, 1]`
+        """
+
+        weight_sampler = self._prepare_weight_sampler()  # prep feature weight distribution
+
+        return feature_decomposition_trajectory(
+            feature_functions=self._feature_functions,
+            weight_sampler=weight_sampler,
+            mean_function=self._mean_function,
+        )
+
+    def update_trajectory(self, trajectory: TrajectoryFunction) -> TrajectoryFunction:
+        """
+        Efficiently update a :const:`TrajectoryFunction` to reflect an update in its
+        underlying :class:`ProbabilisticModel` and resample accordingly.
+
+        For a :class:`FeatureDecompositionTrajectorySampler`, updating the sampler
+        corresponds to resampling the feature functions (taking into account any
+        changed kernel parameters) and recalculating the weight distribution.
+
+        :param trajectory: The trajectory function to be resampled.
+        :return: The new resampled trajectory function.
+        """
+        tf.debugging.Assert(isinstance(trajectory, feature_decomposition_trajectory), [])
+
+        self._feature_functions.resample()  # resample Fourier feature decomposition
+        weight_sampler = self._prepare_weight_sampler()  # recalculate weight distribution
+
+        cast(feature_decomposition_trajectory, trajectory).update(weight_sampler=weight_sampler)
+
+        return trajectory  # return trajectory with updated features and weight distribution
+
+    def resample_trajectory(self, trajectory: TrajectoryFunction) -> TrajectoryFunction:
+        """
+        Efficiently resample a :const:`TrajectoryFunction` in-place to avoid function retracing
+        with every new sample.
+
+        :param trajectory: The trajectory function to be resampled.
+        :return: The new resampled trajectory function.
+        """
+        tf.debugging.Assert(isinstance(trajectory, feature_decomposition_trajectory), [])
+        cast(feature_decomposition_trajectory, trajectory).resample()
+        return trajectory  # return trajectory with resampled weights
+
+    @abstractmethod
+    def _prepare_weight_sampler(self) -> Callable[[int], TensorType]:  # [B] -> [L, B]
+        """
+        Calculate the posterior of the feature weights for the specified feature functions,
+        returning a function that takes in a batch size `B` and returns `B` samples for
+        the weights of each of the `L` features.
+        """
+        raise NotImplementedError
+
+
+class RandomFourierFeatureTrajectorySampler(
+    FeatureDecompositionTrajectorySampler[FeatureDecompositionInternalDataModel]
+):
+    r"""
+    This class builds functions that approximate a trajectory sampled from an underlying Gaussian
+    process model. For tractibility, the Gaussian process is approximated with a Bayesian
+    Linear model across a set of features sampled from the Fourier feature decomposition of
+    the model's kernel. See :cite:`hernandez2014predictive` for details. Currently we do not
+    support models with multiple latent Gaussian processes.
 
     In particular, we approximate the Gaussian processes' posterior samples as the finite feature
     approximation
@@ -242,7 +378,7 @@ class RandomFourierFeatureTrajectorySampler(
 
     def __init__(
         self,
-        model: SupportsGetKernelObservationNoiseInternalData,
+        model: FeatureDecompositionInternalDataModel,
         num_features: int = 1000,
     ):
         """
@@ -254,37 +390,38 @@ class RandomFourierFeatureTrajectorySampler(
         :raise ValueError: If ``dataset`` is empty.
         """
 
-        super().__init__(model)
-
-        if not isinstance(model, SupportsGetKernelObservationNoiseInternalData):
+        if not isinstance(model, FeatureDecompositionInternalDataModel):
             raise NotImplementedError(
-                f"RandomFourierFeatureTrajectorySampler only works with models that support "
-                f"get_kernel, get_observation_noise and get_internal_data; "
+                f"RandomFourierFeatureTrajectorySampler only works with models with "
+                f"get_kernel, get_observation_noise and get_internal_data methods; "
                 f"but received {model.__repr__()}."
             )
 
-        self._model = model
-
         tf.debugging.assert_positive(num_features)
-        self._num_features = num_features  # m
+        self._num_features = num_features
+        self._model = model
+        feature_functions = ResampleableRandomFourierFeatureFunctions(
+            self._model, self._num_features
+        )
+        super().__init__(self._model, feature_functions)
 
-    def __repr__(self) -> str:
-        """"""
-        return f"""{self.__class__.__name__}(
-        {self._model!r},
-        {self._num_features!r})
+    def _prepare_weight_sampler(self) -> Callable[[int], TensorType]:  # [B] -> [L, B]
+        """
+        Calculate the posterior of theta (the feature weights) for the RFFs, returning
+        a function that takes in a batch size `B` and returns `B` samples for
+        the weights of each of the RFF `L` features.
         """
 
-    def _build_theta_posterior(self) -> tfp.distributions.Distribution:
-        # Calculate the posterior of theta (the feature weights) for the specified feature function.
         dataset = self._model.get_internal_data()
         num_data = tf.shape(dataset.query_points)[0]  # n
         if (
             self._num_features < num_data
         ):  # if m < n  then calculate posterior in design space (an m*m matrix inversion)
-            return self._prepare_theta_posterior_in_design_space()
+            theta_posterior = self._prepare_theta_posterior_in_design_space()
         else:  # if n <= m  then calculate posterior in gram space (an n*n matrix inversion)
-            return self._prepare_theta_posterior_in_gram_space()
+            theta_posterior = self._prepare_theta_posterior_in_gram_space()
+
+        return lambda b: theta_posterior.sample(b)
 
     def _prepare_theta_posterior_in_design_space(self) -> tfp.distributions.MultivariateNormalTriL:
         r"""
@@ -304,9 +441,8 @@ class RandomFourierFeatureTrajectorySampler(
         L = tf.linalg.cholesky(D + s)
         D_inv = tf.linalg.cholesky_solve(L, tf.eye(self._num_features, dtype=phi.dtype))
 
-        theta_posterior_mean = tf.matmul(
-            D_inv, tf.matmul(phi, dataset.observations, transpose_a=True)
-        )[
+        residuals = dataset.observations - self._model.get_mean_function()(dataset.query_points)
+        theta_posterior_mean = tf.matmul(D_inv, tf.matmul(phi, residuals, transpose_a=True))[
             :, 0
         ]  # [m,]
         theta_posterior_chol_covariance = tf.linalg.cholesky(
@@ -334,7 +470,10 @@ class RandomFourierFeatureTrajectorySampler(
         s = self._model.get_observation_noise() * tf.eye(num_data, dtype=phi.dtype)
         L = tf.linalg.cholesky(G + s)
         L_inv_phi = tf.linalg.triangular_solve(L, phi)  # [n, m]
-        L_inv_y = tf.linalg.triangular_solve(L, dataset.observations)  # [n, 1]
+        residuals = dataset.observations - self._model.get_mean_function()(
+            dataset.query_points
+        )  # [n, 1]
+        L_inv_y = tf.linalg.triangular_solve(L, residuals)  # [n, 1]
 
         theta_posterior_mean = tf.tensordot(tf.transpose(L_inv_phi), L_inv_y, [[-1], [-2]])[
             :, 0
@@ -348,84 +487,265 @@ class RandomFourierFeatureTrajectorySampler(
             theta_posterior_mean, theta_posterior_chol_covariance
         )
 
-    def get_trajectory(self) -> TrajectoryFunction:
-        """
-        Generate an approximate function draw (trajectory) by sampling weights
-        and evaluating the feature functions.
 
-        :return: A trajectory function representing an approximate trajectory from the Gaussian
-            process, taking an input of shape `[N, D]` and returning shape `[N, 1]`
+class DecoupledTrajectorySampler(
+    FeatureDecompositionTrajectorySampler[
+        Union[
+            FeatureDecompositionInducingPointModel,
+            FeatureDecompositionInternalDataModel,
+        ]
+    ]
+):
+    r"""
+
+    This class builds functions that approximate a trajectory sampled from an underlying Gaussian
+    process model using decoupled sampling. See :cite:`wilson2020efficiently` for an introduction
+    to decoupled sampling. Currently we do not support models with multiple latent Gaussian
+    processes.
+
+    Unlike our :class:`RandomFourierFeatureTrajectorySampler` which uses a RFF decomposition to
+    aprroximate the Gaussian process posterior, a :class:`DecoupledTrajectorySampler` only
+    uses an RFF decomposition to approximate the Gausian process prior and instead using
+    a cannonical decomposition to discretize the effect of updating the prior on the given data.
+
+    In particular, we approximate the Gaussian processes' posterior samples as the finite feature
+    approximation
+
+    .. math:: \hat{f}(.) = \sum_{i=1}^L w_i\phi_i(.) + \sum_{j=1}^m v_jk(.,z_j)
+
+    where :math:`\phi_i(.)` and :math:`w_i` are the Fourier features and their weights that
+    discretize the prior. In contrast, `k(.,z_j)` and :math:`v_i` are the cannonical features and
+    their weights that discretize the data update.
+
+    The expression for :math:`v_i` depends on if we are using an exact Gaussian process or a sparse
+    approximations. See  eq. (13) in :cite:`wilson2020efficiently` for details.
+
+    Note that if a model is both of :class:`FeatureDecompositionInducingPointModel` type and
+    :class:`FeatureDecompositionInternalDataModel` type,
+    :class:`FeatureDecompositionInducingPointModel` will take a priority and inducing points
+    will be used for computations rather than data.
+    """
+
+    def __init__(
+        self,
+        model: Union[
+            FeatureDecompositionInducingPointModel,
+            FeatureDecompositionInternalDataModel,
+        ],
+        num_features: int = 1000,
+    ):
+        """
+        :param sample_size: The desired number of samples.
+        :param model: The model to sample from.
+        :param num_features: The number of features used to approximate the kernel. We use a default
+            of 1000 as it typically perfoms well for a wide range of kernels. Note that very smooth
+            kernels (e.g. RBF) can be well-approximated with fewer features.
+        :raise NotImplementedError: If the model is not of valid type.
+        """
+        if not isinstance(
+            model, (FeatureDecompositionInducingPointModel, FeatureDecompositionInternalDataModel)
+        ):
+            raise NotImplementedError(
+                f"DecoupledTrajectorySampler only works with models that either support "
+                f"get_kernel, get_observation_noise and get_internal_data or support get_kernel "
+                f"and get_inducing_variables; but received {model.__repr__()}."
+            )
+
+        tf.debugging.assert_positive(num_features)
+        self._num_features = num_features
+        self._model = model
+        feature_functions = ResampleableDecoupledFeatureFunctions(self._model, self._num_features)
+
+        super().__init__(self._model, feature_functions)
+
+    def _prepare_weight_sampler(self) -> Callable[[int], TensorType]:
+        """
+        Prepare the sampler function that provides samples of the feature weights
+        for both the RFF and cannonical feature functions, i.e. we return a function
+        that takes in a batch size `B` and returns `B` samples for the weights of each of
+        the `L`  RFF features and `N` cannonical features.
         """
 
-        data_dtype = self._model.get_internal_data().query_points.dtype
+        if isinstance(self._model, FeatureDecompositionInducingPointModel):
+            (  # extract variational parameters
+                inducing_points,
+                q_mu,
+                q_sqrt,
+                whiten,
+            ) = self._model.get_inducing_variables()  # [M, d], [M, 1], [1, M, 1]
+            q_sqrt = q_sqrt[0, :, :]  # [M, M]
+            Kmm = self._model.get_kernel().K(inducing_points, inducing_points)  # [M, M]
+            Kmm += tf.eye(tf.shape(inducing_points)[0], dtype=Kmm.dtype) * DEFAULTS.JITTER
+        else:  # massage quantities from GP to look like variational parameters
+            internal_data = self._model.get_internal_data()
+            inducing_points = internal_data.query_points  # [M, d]
+            q_mu = self._model.get_internal_data().observations  # [M, 1]
+            q_mu = q_mu - self._model.get_mean_function()(
+                inducing_points
+            )  # account for mean function
+            q_sqrt = tf.eye(tf.shape(inducing_points)[0], dtype=tf.float64)  # [M, M]
+            q_sqrt = tf.math.sqrt(self._model.get_observation_noise()) * q_sqrt
+            whiten = False
+            Kmm = (
+                self._model.get_kernel().K(inducing_points, inducing_points) + q_sqrt ** 2
+            )  # [M, M]
 
-        self._feature_functions = RFF(
-            self._model.get_kernel(), self._num_features, dtype=data_dtype
-        )  # prep feature functions at data
-
-        self._theta_posterior = self._build_theta_posterior()  # prep feature weight distribution
-
-        self._feature_functions.b = tf.Variable(
-            self._feature_functions.b
-        )  # store bias as a variable to allow in place updating
-        self._feature_functions.W = tf.Variable(
-            self._feature_functions.W
-        )  # store weights as a variable to allow in place updating
-
-        return fourier_feature_trajectory(
-            feature_functions=self._feature_functions,
-            weight_distribution=self._theta_posterior,
+        tf.debugging.assert_shapes(
+            [
+                (inducing_points, ["M", "d"]),
+                (q_mu, ["M", "1"]),
+                (q_sqrt, ["M", "M"]),
+                (Kmm, ["M", "M"]),
+            ]
         )
 
-    def update_trajectory(self, trajectory: TrajectoryFunction) -> TrajectoryFunction:
+        def weight_sampler(batch_size: int) -> Tuple[TensorType, TensorType]:
+
+            prior_weights = tf.random.normal(  # Non-RFF features will require scaling here
+                [self._num_features, batch_size], dtype=tf.float64
+            )  # [L, B]
+
+            u_noise_sample = tf.matmul(
+                q_sqrt,  # [M, M]
+                tf.random.normal(
+                    (tf.shape(inducing_points)[0], batch_size), dtype=tf.float64
+                ),  # [ M, B]
+            )  # [M, B]
+
+            u_sample = q_mu + u_noise_sample  # [M, B]
+
+            if whiten:
+                Luu = tf.linalg.cholesky(Kmm)  # [M, M]
+                u_sample = tf.matmul(Luu, u_sample)  # [M, B]
+
+            phi_Z = self._feature_functions(inducing_points)[:, : self._num_features]  # [M, B]
+            weight_space_prior_Z = phi_Z @ prior_weights  # [M, B]
+
+            diff = u_sample - weight_space_prior_Z  # [M, B]
+
+            v = compute_A_inv_b(Kmm, diff)  # [M, B]
+
+            tf.debugging.assert_shapes([(v, ["M", "B"]), (prior_weights, ["L", "B"])])
+
+            return tf.transpose(tf.concat([prior_weights, v], axis=0))  # [B, L + M]
+
+        return weight_sampler
+
+
+class ResampleableRandomFourierFeatureFunctions(RFF):  # type: ignore[misc]
+    """
+    A wrapper around GPFlux's random Fourier feature function that allows for
+    efficient in-place updating when generating new decompositions.
+
+    In particular, we store the bias and weights as variables, which can then be
+    updated without triggering expensive graph retracing.
+
+    Note that if a model is both of :class:`FeatureDecompositionInducingPointModel` type and
+    :class:`FeatureDecompositionInternalDataModel` type,
+    :class:`FeatureDecompositionInducingPointModel` will take a priority and inducing points
+    will be used for computations rather than data.
+    """
+
+    def __init__(
+        self,
+        model: Union[
+            FeatureDecompositionInducingPointModel,
+            FeatureDecompositionInternalDataModel,
+        ],
+        n_components: int,
+    ):
         """
-        Efficiently update a :const:`TrajectoryFunction` to reflect an update in its
-        underlying :class:`ProbabilisticModel` and resample accordingly.
-
-        For a :class:`RandomFourierFeatureTrajectorySampler`, updating the sampler
-        corresponds to resampling the feature functions (taking into account any
-        changed kernel parameters) and recalculating the weight distribution.
-
-        :param trajectory: The trajectory function to be resampled.
-        :return: The new resampled trajectory function.
+        :param model: The model that will be approximed by these feature functions.
+        :param n_components: The desired number of features.
+        :raise NotImplementedError: If the model is not of valid type.
         """
-        tf.debugging.Assert(isinstance(trajectory, fourier_feature_trajectory), [])
+        if not isinstance(
+            model,
+            (
+                FeatureDecompositionInducingPointModel,
+                FeatureDecompositionInternalDataModel,
+            ),
+        ):
+            raise NotImplementedError(
+                f"ResampleableRandomFourierFeatureFunctions only work with models that either"
+                f"support get_kernel, get_observation_noise and get_internal_data or support "
+                f"get_kernel and get_inducing_variables;"
+                f"but received {model.__repr__()}."
+            )
 
-        if not hasattr(self._feature_functions, "_bias_init"):
-            # maintain support for gpflux 0.2.3 (but with retracing)
-            return self.get_trajectory()
+        self._kernel = model.get_kernel()
+        self._n_components = n_components
+        super().__init__(self._kernel, self._n_components, dtype=tf.float64)
 
-        bias_shape = tf.shape(self._feature_functions.b)
-        bias_dtype = self._feature_functions.b.dtype
-        weight_shape = tf.shape(self._feature_functions.W)
-        weight_dtype = self._feature_functions.W.dtype
-        self._feature_functions.b.assign(  # resample feature function's bias
-            self._feature_functions._bias_init(bias_shape, dtype=bias_dtype)
-        )
-        self._feature_functions.W.assign(  # resample feature function's weights
-            self._feature_functions._weights_init(weight_shape, dtype=weight_dtype)
+        if isinstance(model, SupportsGetInducingVariables):
+            dummy_X = model.get_inducing_variables()[0][0:1, :]
+        else:
+            dummy_X = model.get_internal_data().query_points[0:1, :]
+
+        self.__call__(dummy_X)  # dummy call to force init of weights
+        self.b: TensorType = tf.Variable(self.b)
+        self.W: TensorType = tf.Variable(self.W)  # allow updateable weights
+
+    def resample(self) -> None:
+        """
+        Resample weights and biases
+        """
+
+        if not hasattr(self, "_bias_init"):
+            # maintain support for gpflux 0.2.3
+            self.b.assign(self._sample_bias(tf.shape(self.b), dtype=self._dtype))
+            self.W.assign(self._sample_weights(tf.shape(self.W), dtype=self._dtype))
+        else:
+            self.b.assign(self._bias_init(tf.shape(self.b), dtype=self._dtype))
+            self.W.assign(self._weights_init(tf.shape(self.W), dtype=self._dtype))
+
+
+class ResampleableDecoupledFeatureFunctions(ResampleableRandomFourierFeatureFunctions):
+    """
+    A wrapper around our :class:`ResampleableRandomFourierFeatureFunctions` which rather
+    than evaluates just `L` RFF functions instead evaluates the concatenation of
+    `L` RFF functions with evaluations of the cannonical basis functions.
+
+    Note that if a model is both of :class:`FeatureDecompositionInducingPointModel` type and
+    :class:`FeatureDecompositionInternalDataModel` type,
+    :class:`FeatureDecompositionInducingPointModel` will take a priority and inducing points
+    will be used for computations rather than data.
+    """
+
+    def __init__(
+        self,
+        model: Union[
+            FeatureDecompositionInducingPointModel,
+            FeatureDecompositionInternalDataModel,
+        ],
+        n_components: int,
+    ):
+        """
+        :param model: The model that will be approximed by these feature functions.
+        :param n_components: The desired number of features.
+        """
+
+        if isinstance(model, SupportsGetInducingVariables):
+            inducing_points = model.get_inducing_variables()[0]  # [M, D]
+        else:
+            inducing_points = model.get_internal_data().query_points  # [M, D]
+
+        self._cannonical_feature_functions = lambda x: tf.linalg.matrix_transpose(
+            model.get_kernel().K(inducing_points, x)
         )
 
-        self._theta_posterior = self._build_theta_posterior()  # recalculate weight distribution
+        super().__init__(model, n_components)
 
-        trajectory.update(weight_distribution=self._theta_posterior)  # type: ignore
-
-        return trajectory  # return trajectory with updated features and weight distribution
-
-    def resample_trajectory(self, trajectory: TrajectoryFunction) -> TrajectoryFunction:
+    def __call__(self, x: TensorType) -> TensorType:  # [N,D] -> [N, L + M]
         """
-        Efficiently resample a :const:`TrajectoryFunction` in-place to avoid function retracing
-        with every new sample.
-
-        :param trajectory: The trajectory function to be resampled.
-        :return: The new resampled trajectory function.
+        combine prior basis functions with cannonical basis functions
         """
-        tf.debugging.Assert(isinstance(trajectory, fourier_feature_trajectory), [])
-        trajectory.resample()  # type: ignore
-        return trajectory  # return trajectory with resampled weights
+        fourier_feature_eval = super().__call__(x)  # [N, L]
+        cannonical_feature_eval = self._cannonical_feature_functions(x)  # [N, M]
+        return tf.concat([fourier_feature_eval, cannonical_feature_eval], axis=-1)  # [N, L + M]
 
 
-class fourier_feature_trajectory(TrajectoryFunctionClass):
+class feature_decomposition_trajectory(TrajectoryFunctionClass):
     r"""
     An approximate sample from a Gaussian processes' posterior samples represented as a
     finite weighted sum of features.
@@ -443,21 +763,24 @@ class fourier_feature_trajectory(TrajectoryFunctionClass):
 
     def __init__(
         self,
-        feature_functions: RFF,
-        weight_distribution: tfp.distributions.MultivariateNormalTriL,
+        feature_functions: Callable[[TensorType], TensorType],
+        weight_sampler: Callable[[int], TensorType],
+        mean_function: Callable[[TensorType], TensorType],
     ):
         """
         :param feature_functions: Set of feature function.
-        :param weight_distribution: Distribution from which feature weights are to be sampled.
+        :param weight_sampler: New sampler that generates feature weight samples.
+        :param mean_function: The underlying model's mean function.
         """
         self._feature_functions = feature_functions
-        self._weight_distribution = weight_distribution
+        self._mean_function = mean_function
+        self._weight_sampler = weight_sampler
         self._initialized = tf.Variable(False)
 
-        num_features = tf.shape(weight_distribution.mean())[0]  # m
-        self._theta_sample = tf.Variable(  # dummy init to be updated before trajectory evaluation
-            tf.ones([0, num_features], dtype=tf.float64), shape=[None, num_features]
-        )  # [0, m]
+        self._weights_sample = tf.Variable(  # dummy init to be updated before trajectory evaluation
+            tf.ones([0, 0], dtype=tf.float64), shape=[None, None]
+        )
+
         self._batch_size = tf.Variable(
             0, dtype=tf.int32
         )  # dummy init to be updated before trajectory evaluation
@@ -474,8 +797,8 @@ class fourier_feature_trajectory(TrajectoryFunctionClass):
         tf.debugging.assert_equal(
             tf.shape(x)[-2],
             self._batch_size.value(),
-            message="""
-            This trajectory only supports batch sizes of {self._batch_size}}.
+            message=f"""
+            This trajectory only supports batch sizes of {self._batch_size}.
             If you wish to change the batch size you must get a new trajectory
             by calling the get_trajectory method of the trajectory sampler.
             """,
@@ -484,23 +807,22 @@ class fourier_feature_trajectory(TrajectoryFunctionClass):
         flat_x, unflatten = flatten_leading_dims(x)  # [N*B, d]
         flattened_feature_evaluations = self._feature_functions(flat_x)  # [N*B, m]
         feature_evaluations = unflatten(flattened_feature_evaluations)  # [N, B, m]
-        return tf.reduce_sum(feature_evaluations * self._theta_sample, -1)  # [N, B]
+        mean = tf.squeeze(self._mean_function(x), -1)  # account for the model's mean function
+        return tf.reduce_sum(feature_evaluations * self._weights_sample, -1) + mean  # [N, B]
 
     def resample(self) -> None:
         """
         Efficiently resample in-place without retracing.
         """
-        self._theta_sample.assign(
-            self._weight_distribution.sample(self._batch_size)
+        self._weights_sample.assign(  # [B, m]
+            self._weight_sampler(self._batch_size)
         )  # resample weights
 
-    def update(self, weight_distribution: tfp.distributions.MultivariateNormalTriL) -> None:
+    def update(self, weight_sampler: Callable[[int], TensorType]) -> None:
         """
         Efficiently update the trajectory with a new weight distribution and resample its weights.
 
-        :param weight_distribution: new distribution from which feature weights are to be sampled.
+        :param weight_sampler: New sampler that generates feature weight samples.
         """
-        self._weight_distribution = weight_distribution  # update weight distribution.
-        self._theta_sample.assign(
-            self._weight_distribution.sample(self._batch_size)
-        )  # resample weights
+        self._weight_sampler = weight_sampler  # update weight sampler
+        self.resample()  # resample weights
