@@ -164,7 +164,7 @@ class SupportsGetObservationNoise(ProbabilisticModel, Protocol):
 
 
 @runtime_checkable
-class SupportsInternalData(ProbabilisticModel, Protocol):
+class SupportsGetInternalData(ProbabilisticModel, Protocol):
     """A probabilistic model that stores and has access to its own training data."""
 
     @abstractmethod
@@ -177,6 +177,22 @@ class SupportsInternalData(ProbabilisticModel, Protocol):
         raise NotImplementedError
 
 
+@runtime_checkable
+class SupportsGetMeanFunction(ProbabilisticModel, Protocol):
+    """A probabilistic model that makes use of a mean function."""
+
+    @abstractmethod
+    def get_mean_function(self) -> Callable[[TensorType], TensorType]:
+        """
+        Return the model's mean function, i.e. a parameterized function that can explain
+        coarse scale variations in the data, leaving just the residuals to be explained by
+        our model.
+
+        :return: The model's mean function.
+        """
+        raise NotImplementedError
+
+
 @runtime_checkable
 class FastUpdateModel(ProbabilisticModel, Protocol):
     """A model with the ability to predict based on (possibly fantasized) supplementary data."""
@@ -248,59 +264,6 @@ class FastUpdateModel(ProbabilisticModel, Protocol):
         )
 
 
-@runtime_checkable
-class EnsembleModel(ProbabilisticModel, Protocol):
-    """
-    This is an interface for ensemble types of models. These models can act as probabilistic models
-    by deriving estimates of epistemic uncertainty from the diversity of predictions made by
-    individual models in the ensemble.
-    """
-
-    @abstractmethod
-    def ensemble_size(self) -> int:
-        """
-        Returns the size of the ensemble, that is, the number of base learners or individual
-        models in the ensemble.
-        """
-        raise NotImplementedError
-
-    @abstractmethod
-    def sample_index(self, size: int) -> TensorType:
-        """
-        Returns indices of individual models in the ensemble sampled randomly with replacement.
-
-        :param size: The number of samples to take.
-        :return: A tensor with indices
-        """
-        raise NotImplementedError
-
-    @abstractmethod
-    def predict_ensemble(self, query_points: TensorType) -> tuple[TensorType, TensorType]:
-        """
-        Returns mean and variance at ``query_points`` for each member of the ensemble. First tensor
-        is the mean and second is the variance, where each has shape [..., M, N, 1], where M is
-        the ``ensemble_size``.
-
-        :param query_points: The points at which to make predictions.
-        :return: The predicted mean and variance of the observations at the specified
-            ``query_points`` for each member of the ensemble.
-        """
-        raise NotImplementedError
-
-    @abstractmethod
-    def sample_ensemble(self, query_points: TensorType, num_samples: int) -> TensorType:
-        """
-        Return ``num_samples`` samples at ``query_points`` where each sample is taken from a
-        distribution given by a randomly chosen model in the ensemble.
-
-        :param query_points: The points at which to sample, with shape [..., N, D].
-        :param num_samples: The number of samples at each point.
-        :return: The samples. For a predictive distribution with event shape E, this has shape
-            [..., S, N] + E, where S is the number of samples.
-        """
-        raise NotImplementedError
-
-
 @runtime_checkable
 class HasTrajectorySampler(ProbabilisticModel, Protocol):
     """A probabilistic model that has an associated trajectory sampler."""
@@ -332,6 +295,15 @@ class HasReparamSampler(ProbabilisticModel, Protocol):
         raise NotImplementedError
 
 
+@runtime_checkable
+class SupportsReparamSamplerObservationNoise(
+    HasReparamSampler, SupportsGetObservationNoise, Protocol
+):
+    """A model that supports both reparam_sampler and get_observation_noise."""
+
+    pass
+
+
 class ModelStack(ProbabilisticModel, Generic[ProbabilisticModelType]):
     r"""
     A :class:`ModelStack` is a wrapper around a number of :class:`ProbabilisticModel`\ s of type
@@ -666,3 +638,20 @@ class TrajectorySampler(ABC, Generic[ProbabilisticModelType]):
         :return: The new trajectory function updated for a new model
         """
         return self.get_trajectory()
+
+
+@runtime_checkable
+class SupportsGetInducingVariables(ProbabilisticModel, Protocol):
+    """A probabilistic model uses and has access to an inducing point approximation."""
+
+    @abstractmethod
+    def get_inducing_variables(self) -> tuple[TensorType, TensorType, TensorType, bool]:
+        """
+        Return the model's inducing variables.
+
+        :return: Tensors containing: the inducing points (i.e. locations of the inducing
+            variables); the variational mean q_mu; the Cholesky decomposition of the
+            variational covariance q_sqrt; and a bool denoting if we are using whitened
+            or not whitened representations.
+        """
+        raise NotImplementedError
