@@ -14,28 +14,26 @@
 
 from __future__ import annotations
 
-from typing import Dict, Optional
+from typing import Any, Dict, Optional
 
+import dill
 import tensorflow as tf
 import tensorflow_probability as tfp
+import tensorflow_probability.python.distributions as tfd
+from tensorflow.python.keras.callbacks import Callback
 
 from ...data import Dataset
 from ...types import TensorType
-from ..interfaces import (
-    EnsembleModel,
-    HasTrajectorySampler,
-    TrainableProbabilisticModel,
-    TrajectorySampler,
-)
+from ..interfaces import HasTrajectorySampler, TrainableProbabilisticModel, TrajectorySampler
 from ..optimizer import KerasOptimizer
-from .architectures import KerasEnsemble
-from .interface import KerasPredictor
-from .sampler import EnsembleTrajectorySampler
-from .utils import negative_log_likelihood, sample_with_replacement
+from .architectures import KerasEnsemble, MultivariateNormalTriL
+from .interface import DeepEnsembleModel, KerasPredictor
+from .sampler import DeepEnsembleTrajectorySampler
+from .utils import negative_log_likelihood, sample_model_index, sample_with_replacement
 
 
 class DeepEnsemble(
-    KerasPredictor, TrainableProbabilisticModel, EnsembleModel, HasTrajectorySampler
+    KerasPredictor, TrainableProbabilisticModel, DeepEnsembleModel, HasTrajectorySampler
 ):
     """
     A :class:`~trieste.model.TrainableProbabilisticModel` wrapper for deep ensembles built using
@@ -64,7 +62,14 @@ class DeepEnsemble(
     can be supplied, as long as it has a Gaussian distribution as a final layer and follows the
     :class:`~trieste.models.keras.KerasEnsembleNetwork` interface.
 
-    Note that currently we do not support setting up the model with dictionary configs and saving
+    A word of caution in case a learning rate scheduler is used in ``fit_args`` to
+    :class:`KerasOptimizer` optimizer instance. Typically one would not want to continue with the
+    reduced learning rate in the subsequent Bayesian optimization step. Hence, we reset the
+    learning rate to the original one after calling the ``fit`` method. In case this is not the
+    behaviour you would like, you will need to subclass the model and overwrite the
+    :meth:`optimize` method.
+
+    Currently we do not support setting up the model with dictionary configs and saving
     the model during Bayesian optimization loop (``track_state`` argument in
     :meth:`~trieste.bayesian_optimizer.BayesianOptimizer.optimize` method should be set to `False`).
     """
@@ -74,6 +79,7 @@ class DeepEnsemble(
         model: KerasEnsemble,
         optimizer: Optional[KerasOptimizer] = None,
         bootstrap: bool = False,
+        diversify: bool = False,
     ) -> None:
         """
         :param model: A Keras ensemble model with probabilistic networks as ensemble members. The
@@ -81,11 +87,18 @@ class DeepEnsemble(
         :param optimizer: The optimizer wrapper with necessary specifications for compiling and
             training the model. Defaults to :class:`~trieste.models.optimizer.KerasOptimizer` with
             :class:`~tf.optimizers.Adam` optimizer, negative log likelihood loss and a dictionary
-            of default arguments for Keras `fit` method: 100 epochs, batch size 100, and verbose 0.
+            of default arguments for Keras `fit` method: 1000 epochs, batch size 16, early stopping
+            callback with patience of 50, and verbose 0.
             See https://keras.io/api/models/model_training_apis/#fit-method for a list of possible
             arguments.
         :param bootstrap: Sample with replacement data for training each network in the ensemble.
             By default set to `False`.
+        :param diversify: Whether to use quantiles from final probabilistic layer as trajectories
+            instead of mean predictions when calling :meth:`trajectory_sampler`. Quantiles are
+            sampled uniformly from a unit interval. This mode can be used to increase the diversity
+            in case of optimizing very large batches of trajectories. When batch size is larger
+            than the ensemble size, multiple quantiles will be used with the same network. By
+            default set to `False`.
         :raise ValueError: If ``model`` is not an instance of
             :class:`~trieste.models.keras.KerasEnsemble` or ensemble has less than two base
             learners (networks).
@@ -98,8 +111,13 @@ class DeepEnsemble(
         if not self.optimizer.fit_args:
             self.optimizer.fit_args = {
                 "verbose": 0,
-                "epochs": 100,
-                "batch_size": 100,
+                "epochs": 1000,
+                "batch_size": 16,
+                "callbacks": [
+                    tf.keras.callbacks.EarlyStopping(
+                        monitor="loss", patience=50, restore_best_weights=True
+                    )
+                ],
             }
 
         if self.optimizer.loss is None:
@@ -110,13 +128,18 @@ class DeepEnsemble(
             loss=[self.optimizer.loss] * model.ensemble_size,
             metrics=[self.optimizer.metrics] * model.ensemble_size,
         )
+        self.original_lr = self.optimizer.optimizer.lr.numpy()
 
         self._model = model
         self._bootstrap = bootstrap
+        self._diversify = diversify
 
     def __repr__(self) -> str:
         """"""
-        return f"DeepEnsemble({self.model!r}, {self.optimizer!r}, {self._bootstrap!r})"
+        return (
+            f"DeepEnsemble({self.model!r}, {self.optimizer!r}, {self._bootstrap!r}, "
+            f"{self._diversify!r})"
+        )
 
     @property
     def model(self) -> tf.keras.Model:
@@ -131,14 +154,12 @@ class DeepEnsemble(
         """
         return self._model.ensemble_size
 
-    def sample_index(self, size: int = 1) -> TensorType:
+    @property
+    def num_outputs(self) -> int:
         """
-        Returns a network index sampled randomly with replacement.
+        Returns the number of outputs trained on by each member network.
         """
-        network_index = tf.random.uniform(
-            shape=(tf.cast(size, tf.int32),), maxval=self.ensemble_size, dtype=tf.int32
-        )
-        return network_index
+        return self._model.num_outputs
 
     def prepare_dataset(
         self, dataset: Dataset
@@ -181,6 +202,17 @@ class DeepEnsemble(
 
         return inputs
 
+    def ensemble_distributions(self, query_points: TensorType) -> tuple[tfd.Distribution, ...]:
+        """
+        Return distributions for each member of the ensemble.
+
+        :param query_points: The points at which to return distributions.
+        :return: The distributions for the observations at the specified
+            ``query_points`` for each member of the ensemble.
+        """
+        x_transformed: dict[str, TensorType] = self.prepare_query_points(query_points)
+        return self._model.model(x_transformed)
+
     def predict(self, query_points: TensorType) -> tuple[TensorType, TensorType]:
         r"""
         Returns mean and variance at ``query_points`` for the whole ensemble.
@@ -208,9 +240,7 @@ class DeepEnsemble(
         :return: The predicted mean and variance of the observations at the specified
             ``query_points``.
         """
-        query_points_transformed = self.prepare_query_points(query_points)
-
-        ensemble_distributions = self.model(query_points_transformed)
+        ensemble_distributions = self.ensemble_distributions(query_points)
         predicted_means = tf.math.reduce_mean(
             [dist.mean() for dist in ensemble_distributions], axis=0
         )
@@ -237,9 +267,7 @@ class DeepEnsemble(
         :return: The predicted mean and variance of the observations at the specified
             ``query_points`` for each member of the ensemble.
         """
-        query_points_transformed = self.prepare_query_points(query_points)
-
-        ensemble_distributions = self.model(query_points_transformed)
+        ensemble_distributions = self.ensemble_distributions(query_points)
         predicted_means = tf.convert_to_tensor([dist.mean() for dist in ensemble_distributions])
         predicted_vars = tf.convert_to_tensor([dist.variance() for dist in ensemble_distributions])
 
@@ -275,29 +303,37 @@ class DeepEnsemble(
         :return: The samples. For a predictive distribution with event shape E, this has shape
             [..., S, N] + E, where S is the number of samples.
         """
-        predicted_means, predicted_vars = self.predict_ensemble(query_points)
+        ensemble_distributions = self.ensemble_distributions(query_points)
+        network_indices = sample_model_index(self.ensemble_size, num_samples)
 
         stacked_samples = []
-        for _ in range(num_samples):
-            network_index = self.sample_index(1)[0]
-            normal = tfp.distributions.Normal(
-                predicted_means[network_index], tf.sqrt(predicted_vars[network_index])
-            )
-            samples = normal.sample()
-            stacked_samples.append(samples)
-
+        for i in range(num_samples):
+            stacked_samples.append(ensemble_distributions[network_indices[i]].sample())
         samples = tf.stack(stacked_samples, axis=0)
+
         return samples  # [num_samples, len(query_points), 1]
 
     def trajectory_sampler(self) -> TrajectorySampler[DeepEnsemble]:
         """
         Return a trajectory sampler. For :class:`DeepEnsemble`, we use an ensemble
         sampler that randomly picks a network from the ensemble and uses its predicted means
-        for generating a trajectory.
+        for generating a trajectory, or optionally randomly sampled quantiles rather than means.
+
+        At the moment only models with single output are supported.
 
         :return: The trajectory sampler.
+        :raise NotImplementedError: If we try to use the sampler with a model that has more than
+            one output.
         """
-        return EnsembleTrajectorySampler(self)
+        if self.num_outputs > 1:
+            raise NotImplementedError(
+                f"""
+                Trajectory sampler does not currently support models with multiple outputs,
+                however received a model with {self.num_outputs} outputs.
+                """
+            )
+
+        return DeepEnsembleTrajectorySampler(self, self._diversify)
 
     def update(self, dataset: Dataset) -> None:
         """
@@ -326,3 +362,42 @@ class DeepEnsemble(
 
         x, y = self.prepare_dataset(dataset)
         self.model.fit(x=x, y=y, **self.optimizer.fit_args)
+
+        # Reset lr in case there was an lr schedule: a schedule will have change the learning rate,
+        # so that the next time we call `optimize` the starting learning rate would be different.
+        # Therefore we make sure the learning rate is set back to its initial value.
+        self.optimizer.optimizer.lr.assign(self.original_lr)
+
+    def __getstate__(self) -> dict[str, Any]:
+        # When pickling use to_json to save any optimizer fit_arg callback models
+        state = self.__dict__.copy()
+        if self._optimizer:
+            # jsonify all the callback models, pickle the optimizer(!), and revert (ugh!)
+            callback: Callback
+            saved_models: list[KerasOptimizer] = []
+            for callback in self._optimizer.fit_args["callbacks"]:
+                saved_models.append(callback.model)
+                callback.model = callback.model and callback.model.to_json()
+            state["_optimizer"] = dill.dumps(state["_optimizer"])
+            for callback, model in zip(self._optimizer.fit_args["callbacks"], saved_models):
+                callback.model = model
+        return state
+
+    def __setstate__(self, state: dict[str, Any]) -> None:
+        # Restore optimizer and callback models after depickling, and recompile.
+        self.__dict__.update(state)
+        if self._optimizer:
+            # unpickle the optimizer, and restore all the callback models
+            self._optimizer = dill.loads(self._optimizer)
+            for callback in self._optimizer.fit_args.get("callbacks", []):
+                if callback.model:
+                    callback.model = tf.keras.models.model_from_json(
+                        callback.model,
+                        custom_objects={"MultivariateNormalTriL": MultivariateNormalTriL},
+                    )
+        # Recompile the model
+        self._model.model.compile(
+            self.optimizer.optimizer,
+            loss=[self.optimizer.loss] * self._model.ensemble_size,
+            metrics=[self.optimizer.metrics] * self._model.ensemble_size,
+        )
